@@ -32,8 +32,6 @@
 #include <QtGui/QRadialGradient>
 #include <QtGui/QIcon>
 #include <QtCore/QTimer>
-#include <QCompleter>
-#include <QFileSystemModel>
 #include <QtCore/QUrl>
 #include <QFontDialog>
 #include <QFileDialog>
@@ -43,7 +41,6 @@
 #include <KCodecAction>
 
 #include <KIconDialog>
-#include <KUrlCompletion>
 #include <KWindowSystem>
 #include <KMessageBox>
 #include <KLocalizedString>
@@ -267,34 +264,8 @@ void EditProfileDialog::setupGeneralPage(const Profile::Ptr profile)
 
     ShellCommand command(profile->command() , profile->arguments());
     _ui->commandEdit->setText(command.fullCommand());
-
-#pragma message("Look at this setCompletionObject again")
-//    KUrlCompletion* exeCompletion = new KUrlCompletion(KUrlCompletion::ExeCompletion);
-//    exeCompletion->setParent(this);
-//    exeCompletion->setDir(QUrl());
-//    _ui->commandEdit->setCompletionObject(exeCompletion);
-
-/* The below causes a noticeable delay when opening the dialog - I'm not entirely sure
-   this is the best way to handle this.
-   Issue is that QLineEdit->SetCompleter() won't work w/ KDE's KUrlCompletion
-
-    QFileSystemModel *commandEditDirModel = new QFileSystemModel(this);
-    commandEditDirModel->setFilter(QDir::AllEntries);
-    QFileInfo commandFileInfo(profile->command());
-    // If command is /usr/bin/zsh, start at /usr/bin for completion
-    commandEditDirModel->setRootPath(commandFileInfo.absolutePath());
-    QCompleter *commandEditCompleter = new QCompleter(this);
-    commandEditCompleter->setModel(commandEditDirModel);
-    _ui->commandEdit->setCompleter(commandEditCompleter);
-
-    QFileSystemModel *initialEditDirModel = new QFileSystemModel(this);
-    initialEditDirModel->setFilter(QDir::AllEntries);
-    initialEditDirModel->setRootPath(QString('/'));
-    _ui->initialDirEdit->setText(profile->defaultWorkingDirectory());
-    QCompleter *initialDirCompleter = new QCompleter(this);
-    initialDirCompleter->setModel(initialEditDirModel);
-    _ui->initialDirEdit->setCompleter(initialDirCompleter);
-*/
+    // If a "completion" is requested, consider changing this to KLineEdit
+    // and using KCompletion.
     _ui->initialDirEdit->setText(profile->defaultWorkingDirectory());
     _ui->initialDirEdit->setClearButtonEnabled(true);
 
@@ -463,6 +434,10 @@ void EditProfileDialog::setupAppearancePage(const Profile::Ptr profile)
     _ui->fontPreviewLabel->setFont(profileFont);
     setFontInputValue(profileFont);
 
+    // Always set to unchecked
+    _ui->showAllFontsButton->setChecked(false);
+    connect(_ui->showAllFontsButton, &QCheckBox::toggled, this, &Konsole::EditProfileDialog::showAllFontsButtonWarning);
+
     connect(_ui->fontSizeInput, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged), this, &Konsole::EditProfileDialog::setFontSize);
     connect(_ui->selectFontButton, &QPushButton::clicked, this, &Konsole::EditProfileDialog::showFontDialog);
 
@@ -472,9 +447,23 @@ void EditProfileDialog::setupAppearancePage(const Profile::Ptr profile)
 
     _ui->boldIntenseButton->setChecked(profile->boldIntense());
     connect(_ui->boldIntenseButton, &QCheckBox::toggled, this, &Konsole::EditProfileDialog::setBoldIntense);
+
+    _ui->useFontLineCharactersButton->setChecked(profile->useFontLineCharacters());
+    connect(_ui->useFontLineCharactersButton, &QCheckBox::toggled, this, &Konsole::EditProfileDialog::useFontLineCharacters);
+
     _ui->enableMouseWheelZoomButton->setChecked(profile->mouseWheelZoomEnabled());
     connect(_ui->enableMouseWheelZoomButton, &QCheckBox::toggled, this, &Konsole::EditProfileDialog::toggleMouseWheelZoom);
 }
+
+void EditProfileDialog::showAllFontsButtonWarning(bool enable)
+{
+    if (enable) {
+        KMessageBox::information(this,
+            "By its very nature, a terminal program requires font characters that are equal width (monospace).  Any non monospaced font may cause display issues.  This should not be necessary except in rare cases.",
+            "Warning");
+    }
+}
+
 void EditProfileDialog::setAntialiasText(bool enable)
 {
     QFont profileFont = _ui->fontPreviewLabel->font();
@@ -489,6 +478,11 @@ void EditProfileDialog::setBoldIntense(bool enable)
     preview(Profile::BoldIntense, enable);
     updateTempProfileProperty(Profile::BoldIntense, enable);
 }
+void EditProfileDialog::useFontLineCharacters(bool enable)
+{
+    preview(Profile::UseFontLineCharacters, enable);
+    updateTempProfileProperty(Profile::UseFontLineCharacters, enable);
+}
 void EditProfileDialog::toggleMouseWheelZoom(bool enable)
 {
     updateTempProfileProperty(Profile::MouseWheelZoomEnabled, enable);
@@ -1070,7 +1064,12 @@ void EditProfileDialog::setupMousePage(const Profile::Ptr profile)
 }
 void EditProfileDialog::setupAdvancedPage(const Profile::Ptr profile)
 {
-    BooleanOption  options[] = { {
+    BooleanOption  options[] = {
+        {
+            _ui->enableUrlHints , Profile::EnableUrlHints ,
+            SLOT(toggleEnableUrlHints(bool))
+        },
+        {
             _ui->enableBlinkingTextButton , Profile::BlinkingTextEnabled ,
             SLOT(toggleBlinkingText(bool))
         },
@@ -1198,6 +1197,10 @@ void EditProfileDialog::TripleClickModeChanged(int newValue)
 {
     updateTempProfileProperty(Profile::TripleClickMode, newValue);
 }
+void EditProfileDialog::toggleEnableUrlHints(bool enable)
+{
+    updateTempProfileProperty(Profile::EnableUrlHints, enable);
+}
 void EditProfileDialog::toggleBlinkingText(bool enable)
 {
     updateTempProfileProperty(Profile::BlinkingTextEnabled, enable);
@@ -1220,11 +1223,17 @@ void EditProfileDialog::fontSelected(const QFont& aFont)
 void EditProfileDialog::showFontDialog()
 {
     QFont currentFont = _ui->fontPreviewLabel->font();
+    bool showAllFonts = _ui->showAllFontsButton->isChecked();
 
     bool result;
-    currentFont = QFontDialog::getFont(&result, currentFont, this,
+    if (showAllFonts) {
+        currentFont = QFontDialog::getFont(&result, currentFont, this,
+                                       i18n("Select All Font"));
+    } else {
+        currentFont = QFontDialog::getFont(&result, currentFont, this,
                                        i18n("Select Fixed Width Font"),
                                        QFontDialog::MonospacedFonts);
+    }
     if (!result) return;
 
     fontSelected(currentFont);
