@@ -123,8 +123,10 @@ private slots:
     // appearance page
     void setFontSize(double pointSize);
     void setFontInputValue(const QFont&);
+    void showAllFontsButtonWarning(bool enable);
     void setAntialiasText(bool enable);
     void setBoldIntense(bool enable);
+    void useFontLineCharacters(bool enable);
     void showFontDialog();
     void newColorScheme();
     void editColorScheme();
@@ -172,6 +174,7 @@ private slots:
     void togglebidiRendering(bool);
     void lineSpacingChanged(int);
     void toggleBlinkingCursor(bool);
+    void toggleEnableUrlHints(bool);
 
     void setCursorShape(int);
     void autoCursorColor();
