@@ -45,12 +45,13 @@
 #include <KUser>
 #include <QDebug>
 
-#if defined(Q_OS_FREEBSD) || defined(Q_OS_OPENBSD) || defined(Q_OS_MAC)
+#if defined(Q_OS_FREEBSD) || defined(Q_OS_OPENBSD) || defined(Q_OS_OSX)
 #include <sys/sysctl.h>
 #endif
 
-#if defined(Q_OS_MAC)
+#if defined(Q_OS_OSX)
 #include <libproc.h>
+#include <qplatformdefs.h>
 #endif
 
 #if defined(Q_OS_FREEBSD) || defined(Q_OS_OPENBSD)
@@ -59,6 +60,9 @@
 #include <sys/syslimits.h>
 #   if defined(Q_OS_FREEBSD)
 #   include <libutil.h>
+#   include <sys/param.h>
+#   include <sys/queue.h>
+#   include <libprocstat.h>
 #   endif
 #endif
 
@@ -80,6 +84,7 @@ ProcessInfo::ProcessInfo(int aPid , bool enableEnvironmentRead)
     , _lastError(NoError)
     , _userName(QString())
     , _userHomeDir(QString())
+    , _userNameRequired(true)
 {
 }
 
@@ -106,7 +111,8 @@ QString ProcessInfo::validCurrentDir() const
     int currentPid = parentPid(&ok);
     QString dir = currentDir(&ok);
     while (!ok && currentPid != 0) {
-        ProcessInfo* current = ProcessInfo::newInstance(currentPid);
+        QString tmp = QString("");
+        ProcessInfo* current = ProcessInfo::newInstance(currentPid, tmp);
         current->update();
         currentPid = current->parentPid(&ok);
         dir = current->currentDir(&ok);
@@ -296,6 +302,16 @@ void ProcessInfo::setForegroundPid(int aPid)
     _fields |= FOREGROUND_PID;
 }
 
+void ProcessInfo::setUserNameRequired(bool need)
+{
+    _userNameRequired = need;
+}
+
+bool ProcessInfo::userNameRequired() const
+{
+    return _userNameRequired;
+}
+
 QString ProcessInfo::currentDir(bool* ok) const
 {
     if (ok)
@@ -348,7 +364,7 @@ void ProcessInfo::setFileError(QFile::FileError error)
 // implementations of the UnixProcessInfo abstract class.
 //
 
-NullProcessInfo::NullProcessInfo(int aPid, bool enableEnvironmentRead)
+NullProcessInfo::NullProcessInfo(int aPid, const QString& /*titleFormat*/, bool enableEnvironmentRead)
     : ProcessInfo(aPid, enableEnvironmentRead)
 {
 }
@@ -363,9 +379,10 @@ void NullProcessInfo::readUserName()
 }
 
 #if !defined(Q_OS_WIN)
-UnixProcessInfo::UnixProcessInfo(int aPid, bool enableEnvironmentRead)
+UnixProcessInfo::UnixProcessInfo(int aPid, const QString& titleFormat, bool enableEnvironmentRead)
     : ProcessInfo(aPid, enableEnvironmentRead)
 {
+    setUserNameRequired(titleFormat.contains(QLatin1String("%u")));
 }
 
 bool UnixProcessInfo::readProcessInfo(int aPid , bool enableEnvironmentRead)
@@ -419,8 +436,8 @@ void UnixProcessInfo::readUserName()
 class LinuxProcessInfo : public UnixProcessInfo
 {
 public:
-    LinuxProcessInfo(int aPid, bool env) :
-        UnixProcessInfo(aPid, env) {
+    LinuxProcessInfo(int aPid, const QString& titleFormat, bool env) :
+        UnixProcessInfo(aPid, titleFormat, env) {
     }
 
 private:
@@ -462,7 +479,10 @@ private:
             const int uid = uidString.toInt(&ok);
             if (ok)
                 setUserId(uid);
-            readUserName();
+            // This will cause constant opening of /etc/passwd
+            if (userNameRequired()) {
+                readUserName();
+            }
         } else {
             setFileError(statusInfo.error());
             return false;
@@ -611,8 +631,8 @@ private:
 class FreeBSDProcessInfo : public UnixProcessInfo
 {
 public:
-    FreeBSDProcessInfo(int aPid, bool readEnvironment) :
-        UnixProcessInfo(aPid, readEnvironment) {
+    FreeBSDProcessInfo(int aPid, const QString& titleFormat, bool readEnvironment) :
+        UnixProcessInfo(aPid, titleFormat, readEnvironment) {
     }
 
 private:
@@ -663,26 +683,68 @@ private:
 
         managementInfoBase[0] = CTL_KERN;
         managementInfoBase[1] = KERN_PROC;
-        managementInfoBase[2] = KERN_PROC_PID;
+        managementInfoBase[2] = KERN_PROC_ARGS;
         managementInfoBase[3] = aPid;
 
         len = sizeof(args);
         if (sysctl(managementInfoBase, 4, args, &len, NULL, 0) == -1)
             return false;
 
-        const QStringList& argumentList = QString(args).split(QChar('\0'));
-
-        for (QStringList::const_iterator it = argumentList.begin(); it != argumentList.end(); ++it) {
-            addArgument(*it);
+        // len holds the length of the string
+        QString qargs = QString::fromLocal8Bit(args,len);
+        foreach (QString value, qargs.split('\u0000'))
+        {
+            if (!value.isEmpty())
+            {
+                addArgument(value);
+            }
         }
 
         return true;
     }
 
     virtual bool readEnvironment(int aPid) {
-        Q_UNUSED(aPid);
-        // Not supported in FreeBSD?
-        return false;
+
+        struct procstat *prstat = procstat_open_sysctl();
+        if (prstat == nullptr) {
+            return false;
+        }
+
+        kinfo_proc *procinfo;
+        unsigned int cnt;
+        procinfo = procstat_getprocs(prstat, KERN_PROC_PID, aPid, &cnt);
+        if (procinfo == nullptr || cnt != 1)
+        {
+            procstat_close(prstat);
+            return false;
+        }
+
+        // pass 0, as the third argument, as we want to have every environment
+        // variable defined -- code courtesy of procstats procstats_arg.c
+        char **envs = procstat_getenvv(prstat, procinfo, 0);
+        if (envs == nullptr)
+        {
+            procstat_close(prstat);
+            return false;
+        }
+        int i;
+        QString name, value;
+        for (i = 0; envs[i] != nullptr; i++)
+        {
+            QString entry = QString::fromLocal8Bit(envs[i]);
+            const int splitPos = entry.indexOf('=');
+
+            if (splitPos != -1) {
+                name = entry.mid(0, splitPos);
+                value = entry.mid(splitPos + 1, -1);
+
+                addEnvironmentBinding(name, value);
+            }
+        }
+
+        procstat_freeenvv(prstat);
+        procstat_close(prstat);
+        return true;
     }
 
     virtual bool readCurrentDir(int aPid) {
@@ -731,8 +793,8 @@ private:
 class OpenBSDProcessInfo : public UnixProcessInfo
 {
 public:
-    OpenBSDProcessInfo(int aPid, bool readEnvironment) :
-        UnixProcessInfo(aPid, readEnvironment) {
+    OpenBSDProcessInfo(int aPid, const QString& titleFormat, bool readEnvironment) :
+        UnixProcessInfo(aPid, titleFormat, readEnvironment) {
     }
 
 private:
@@ -861,12 +923,12 @@ private:
     }
 };
 
-#elif defined(Q_OS_MAC)
+#elif defined(Q_OS_OSX)
 class MacProcessInfo : public UnixProcessInfo
 {
 public:
-    MacProcessInfo(int aPid, bool env) :
-        UnixProcessInfo(aPid, env) {
+    MacProcessInfo(int aPid, const QString& titleFormat, bool env) :
+        UnixProcessInfo(aPid, titleFormat, env) {
     }
 
 private:
@@ -874,8 +936,7 @@ private:
         int managementInfoBase[4];
         size_t mibLength;
         struct kinfo_proc* kInfoProc;
-/*
-        KDE_struct_stat statInfo;
+        QT_STATBUF statInfo;
 
         // Find the tty device of 'pid' (Example: /dev/ttys001)
         managementInfoBase[0] = CTL_KERN;
@@ -898,7 +959,7 @@ private:
                 const QByteArray deviceName = fullDeviceName.toLatin1();
                 const char* ttyName = deviceName.data();
 
-                if (KDE::stat(ttyName, &statInfo) != 0)
+                if (QT_STAT(ttyName, &statInfo) != 0)
                     return false;
 
                 // Find all processes attached to ttyName
@@ -923,8 +984,6 @@ private:
             setPid(aPid);
         }
         return true;
-*/
-        return false;
     }
 
     virtual bool readArguments(int aPid) {
@@ -961,8 +1020,8 @@ private:
 class SolarisProcessInfo : public UnixProcessInfo
 {
 public:
-    SolarisProcessInfo(int aPid, bool readEnvironment)
-        : UnixProcessInfo(aPid, readEnvironment) {
+    SolarisProcessInfo(int aPid, const QString& titleFormat, bool readEnvironment)
+        : UnixProcessInfo(aPid, titleFormat, readEnvironment) {
     }
 private:
     virtual bool readProcInfo(int aPid) {
@@ -1018,6 +1077,10 @@ private:
 
 SSHProcessInfo::SSHProcessInfo(const ProcessInfo& process)
     : _process(process)
+    , _user(QString())
+    , _host(QString())
+    , _port(QString())
+    , _command(QString())
 {
     bool ok = false;
 
@@ -1107,8 +1170,17 @@ SSHProcessInfo::SSHProcessInfo(const ProcessInfo& process)
                     _host = args[i];
                 }
             } else {
-                // host has already been found, this must be the command argument
-                _command = args[i];
+                // host has already been found, this must be part of the
+                // command arguments.
+                // Note this is not 100% correct.  If any of the above
+                // noArgumentOptions or singleArgumentOptions are found, this
+                // will not be correct (example ssh server top -i 50)
+                // Suggest putting ssh command in quotes
+                if (_command.isEmpty()) {
+                    _command = args[i];
+                } else {
+                    _command = _command + QString(' ') + args[i];
+                }
             }
         }
     } else {
@@ -1153,6 +1225,16 @@ QString SSHProcessInfo::format(const QString& input) const
     // search for and replace known markers
     output.replace(QLatin1String("%u"), _user);
 
+    // provide 'user@' if user is defined -- this makes nicer
+    // remote tabs possible: "%U%h %c" => User@Host Command
+    //                                 => Host Command
+    // Depending on whether -l was passed to ssh (which is mostly not the
+    // case due to ~/.ssh/config).
+    if (_user.isEmpty())
+        output.replace(QLatin1String("%U"), QString());
+    else
+        output.replace(QLatin1String("%U"), _user + '@');
+
     if (isIpAddress)
         output.replace(QLatin1String("%h"), _host);
     else
@@ -1164,20 +1246,20 @@ QString SSHProcessInfo::format(const QString& input) const
     return output;
 }
 
-ProcessInfo* ProcessInfo::newInstance(int aPid, bool enableEnvironmentRead)
+ProcessInfo* ProcessInfo::newInstance(int aPid, const QString& titleFormat, bool enableEnvironmentRead)
 {
 #if defined(Q_OS_LINUX)
-    return new LinuxProcessInfo(aPid, enableEnvironmentRead);
+    return new LinuxProcessInfo(aPid, titleFormat, enableEnvironmentRead);
 #elif defined(Q_OS_SOLARIS)
-    return new SolarisProcessInfo(aPid, enableEnvironmentRead);
-#elif defined(Q_OS_MAC)
-    return new MacProcessInfo(aPid, enableEnvironmentRead);
+    return new SolarisProcessInfo(aPid, titleFormat, enableEnvironmentRead);
+#elif defined(Q_OS_OSX)
+    return new MacProcessInfo(aPid, titleFormat, enableEnvironmentRead);
 #elif defined(Q_OS_FREEBSD)
-    return new FreeBSDProcessInfo(aPid, enableEnvironmentRead);
+    return new FreeBSDProcessInfo(aPid, titleFormat, enableEnvironmentRead);
 #elif defined(Q_OS_OPENBSD)
-    return new OpenBSDProcessInfo(aPid, enableEnvironmentRead);
+    return new OpenBSDProcessInfo(aPid, titleFormat, enableEnvironmentRead);
 #else
-    return new NullProcessInfo(aPid, enableEnvironmentRead);
+    return new NullProcessInfo(aPid, titleFormat, enableEnvironmentRead);
 #endif
 }
 
