@@ -223,6 +223,9 @@ public:
         /** (bool) Whether character with intense colors should be rendered
          * in bold font or just in bright color. */
         BoldIntense,
+        /** (bool) Whether to use font's line characters instead of the
+         * builtin code. */
+        UseFontLineCharacters,
         /** (bool) Whether new sessions should be started in the same
          * directory as the currently active session.
          */
@@ -248,7 +251,11 @@ public:
         /** (bool) If true, mouse wheel scroll with Ctrl key pressed
          * increases/decreases the terminal font size.
          */
-        MouseWheelZoomEnabled
+        MouseWheelZoomEnabled,
+        /** (bool) Enables showing a hint for all visible URLs
+         * which allows activating URLs with the keyboard.
+         */
+        EnableUrlHints
     };
 
     /**
@@ -475,6 +482,11 @@ public:
         return property<bool>(Profile::BoldIntense);
     }
 
+    /** Convenience method for property<bool>(Profile::UseFontLineCharacters)*/
+    bool useFontLineCharacters() const {
+        return property<bool>(Profile::UseFontLineCharacters);
+    }
+
     /** Convenience method for property<bool>(Profile::StartInCurrentSessionDir) */
     bool startInCurrentSessionDir() const {
         return property<bool>(Profile::StartInCurrentSessionDir);
