@@ -822,13 +822,18 @@ bool Session::closeInNormalWay()
         return true;
     }
 
-    _shellProcess->sendEof();
+    static QSet<QString> knownShells({"ash", "bash", "csh", "dash", "fish", "hush", "ksh", "mksh", "pdksh", "tcsh", "zsh"});
 
-    if (_shellProcess->waitForFinished(1000)) {
-        return true;
+    // If only the session's shell is running, try sending an EOF for a clean exit
+    if (!isForegroundProcessActive() && knownShells.contains(QFileInfo(_program).fileName())) {
+        _shellProcess->sendEof();
+
+        if (_shellProcess->waitForFinished(1000)) {
+            return true;
+        }
+        qWarning() << "shell did not close, sending SIGHUP";
     }
 
-    qWarning() << "shell did not close, sending SIGHUP";
 
     // We tried asking nicely, ask a bit less nicely
     if (kill(SIGHUP)) {
@@ -987,6 +992,9 @@ ProcessInfo* Session::getProcessInfo()
     ProcessInfo* process = 0;
 
     if (isForegroundProcessActive()) {
+        if (_foregroundProcessInfo == nullptr) {
+            updateForegroundProcessInfo();
+        }
         process = _foregroundProcessInfo;
     } else {
         updateSessionProcessInfo();
@@ -1464,7 +1472,7 @@ int Session::foregroundProcessId()
 bool Session::isForegroundProcessActive()
 {
     // foreground process info is always updated after this
-    return updateForegroundProcessInfo() && (processId() != _foregroundPid);
+    return (_shellProcess->pid() != _shellProcess->foregroundProcessGroup());
 }
 
 QString Session::foregroundProcessName()
