@@ -36,6 +36,7 @@
 
 // Konsole
 #include "konsoleprivate_export.h"
+#include "config-konsole.h"
 
 class QColor;
 
@@ -55,7 +56,7 @@ class HistoryType;
  * Platform-specific main shortcut "opcode":
  */
 enum Modifier {
-#ifdef Q_OS_MAC
+#ifdef Q_OS_OSX
     ACCEL = Qt::META
 #else
     ACCEL = Qt::CTRL
@@ -351,6 +352,7 @@ public:
         IconNameAndWindowTitle = 0,
         IconName               = 1,
         WindowTitle            = 2,
+        CurrentDirectory       = 7,   // From VTE (supposedly 6 was for dir, 7 for file, but whatever)
         TextColor              = 10,
         BackgroundColor        = 11,
         SessionName            = 30,  // Non-standard
@@ -472,13 +474,24 @@ public slots:
 
     /**
      * Sends @p text to the current foreground terminal program.
+     * @param eol send this after @p text
      */
+    void sendTextToTerminal(const QString& text, const QChar& eol = QChar()) const;
+
+#if defined(REMOVE_SENDTEXT_RUNCOMMAND_DBUS_METHODS)
+    void sendText(const QString& text) const;
+#else
     Q_SCRIPTABLE void sendText(const QString& text) const;
+#endif
 
     /**
      * Sends @p command to the current foreground terminal program.
      */
+#if defined(REMOVE_SENDTEXT_RUNCOMMAND_DBUS_METHODS)
+    void runCommand(const QString& command) const;
+#else
     Q_SCRIPTABLE void runCommand(const QString& command) const;
+#endif
 
     /**
      * Sends a mouse event of type @p eventType emitted by button
@@ -663,8 +676,8 @@ signals:
     void selectionChanged(const QString& text);
 
     /**
-     * Emitted when background request ('\033]11;?\a') terminal code received.
-     * Terminal is expected send '\033]11;rgb:RRRR/GGGG/BBBB\a' response.
+     * Emitted when background request ("\033]11;?\a") terminal code received.
+     * Terminal is expected send "\033]11;rgb:RRRR/GGGG/BBBB\a" response.
      *
      * Originally implemented to support vim's background detection feature
      * (without explictly setting 'bg=dark' within local/remote vimrc)
@@ -755,6 +768,7 @@ private:
 
     QString        _initialWorkingDir;
     QString        _currentWorkingDir;
+    QUrl           _reportedWorkingUrl;
 
     ProcessInfo*   _sessionProcessInfo;
     ProcessInfo*   _foregroundProcessInfo;
