@@ -27,6 +27,7 @@
 #include <QtCore/QPointer>
 #include <QtCore/QString>
 #include <QtCore/QHash>
+#include <QtCore/QRegularExpression>
 
 // KDE
 #include <KXMLGUIClient>
@@ -59,7 +60,6 @@ class ScreenWindow;
 class TerminalDisplay;
 class IncrementalSearchBar;
 class ProfileList;
-class UrlFilter;
 class RegExpFilter;
 class EditProfileDialog;
 
@@ -270,7 +270,6 @@ private slots:
     // to take a snapshot of the state of the
     // foreground process in the terminal
 
-    void requireUrlFilterUpdate();
     void highlightMatches(bool highlight);
     void scrollBackOptionsChanged(int mode , int lines);
     void sessionResizeRequest(const QSize& size);
@@ -296,7 +295,7 @@ private:
     // direction - value from SearchHistoryTask::SearchDirection enum to specify
     //             the search direction
     void beginSearch(const QString& text , int direction);
-    QRegExp regexpFromSearchBarOptions();
+    QRegularExpression regexpFromSearchBarOptions() const;
     bool reverseSearchChecked() const;
     void setupCommonActions();
     void setupExtraActions();
@@ -317,7 +316,6 @@ private:
     QString    _sessionIconName;
     int        _previousState;
 
-    UrlFilter*      _viewUrlFilter;
     RegExpFilter*   _searchFilter;
 
     QAction* _copyInputToAllTabsAction;
@@ -328,8 +326,6 @@ private:
 
     QTimer* _interactionTimer;
 
-    bool _urlFilterUpdateRequired;
-
     int _searchStartLine;
     int _prevSearchResultLine;
     QPointer<IncrementalSearchBar> _searchBar;
@@ -354,7 +350,7 @@ private:
     QStringList _bookmarkValidProgramsToClear;
 
     bool _isSearchBarEnabled;
-    QWeakPointer<EditProfileDialog> _editProfileDialog;
+    QPointer<EditProfileDialog> _editProfileDialog;
 
     QString _searchText;
 };
@@ -498,9 +494,9 @@ public:
     void addScreenWindow(Session* session , ScreenWindow* searchWindow);
 
     /** Sets the regular expression which is searched for when execute() is called */
-    void setRegExp(const QRegExp& regExp);
+    void setRegExp(const QRegularExpression &regExp);
     /** Returns the regular expression which is searched for when execute() is called */
-    QRegExp regExp() const;
+    QRegularExpression regExp() const;
 
     /** Specifies the direction to search in when execute() is called. */
     void setSearchDirection(SearchDirection direction);
@@ -529,7 +525,7 @@ private:
     void highlightResult(ScreenWindowPtr window , int position);
 
     QMap< SessionPtr , ScreenWindowPtr > _windows;
-    QRegExp _regExp;
+    QRegularExpression _regExp;
     SearchDirection _direction;
     int _startLine;
 
